package de.firstdata.ipg.api.client.action.datastorage.object;

import de.firstdata.ipg.api.client.action.datastorage.StoreHostedData;
import de.firstdata.ipg.api.client.transaction.creditcard.CreditCard;
import de.firstdata.ipg.api.client.transaction.dedirectdebit.DEDirectDebit;
import de.firstdata.ipg.api.schema.CardFunctionType;
import de.firstdata.ipg.api.schema.CreditCardData;
import de.firstdata.ipg.api.schema.DEDirectDebitData;
import de.firstdata.ipg.api.schema.DataStorageItem;
import de.firstdata.ipg.api.schema.TokenType;

public class StoreHostedDataObject extends HostedDataObject implements StoreHostedData {

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard) {
		getItem(hostedDataID).setCreditCardData(getCreditCardData(creditCard));
	}

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard, final CardFunctionType cardFunction) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setCreditCardData(getCreditCardData(creditCard));
		item.setCardFunction(cardFunction);
	}

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard, final boolean declineHostedDataDuplicates) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setCreditCardData(getCreditCardData(creditCard));
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
	}

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard, final boolean declineHostedDataDuplicates, final CardFunctionType cardFunction) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setCreditCardData(getCreditCardData(creditCard));
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
		item.setCardFunction(cardFunction);
	}

	@Override
	public void add(final String hostedDataID, final DEDirectDebit account, final String billingName) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setDEDirectDebitData(getDEDirectDebitData(account));
		item.setBillingName(billingName);
	}

	@Override
	public void add(final String hostedDataID, final DEDirectDebit account, final boolean declineHostedDataDuplicates) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setDEDirectDebitData(getDEDirectDebitData(account));
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
	}

	@Override
	public void add(final String hostedDataID, final String orderID) {
		getItem(hostedDataID).setOrderId(orderID);
	}

	@Override
	public void add(final String hostedDataID, final String orderID, final boolean declineHostedDataDuplicates) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setOrderId(orderID);
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
	}

	// ----------------------------

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard, final TokenType tokenType, final Boolean assignToken) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setCreditCardData(getCreditCardData(creditCard));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
	}

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard, final boolean declineHostedDataDuplicates, final TokenType tokenType,
			final Boolean assignToken) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setCreditCardData(getCreditCardData(creditCard));
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
	}

	@Override
	public void add(final String hostedDataID, final CreditCard creditCard, final boolean declineHostedDataDuplicates, final TokenType tokenType,
			final Boolean assignToken, final CardFunctionType cardFunction) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setCreditCardData(getCreditCardData(creditCard));
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
		item.setCardFunction(cardFunction);
	}

	@Override
	public void add(final String hostedDataID, final DEDirectDebit account, final TokenType tokenType, final Boolean assignToken) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setDEDirectDebitData(getDEDirectDebitData(account));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
	}

	@Override
	public void add(final String hostedDataID, final DEDirectDebit account, final boolean declineHostedDataDuplicates, final TokenType tokenType,
			final Boolean assignToken) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setDEDirectDebitData(getDEDirectDebitData(account));
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
	}

	@Override
	public void add(final String hostedDataID, final String orderID, final TokenType tokenType, final Boolean assignToken) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setOrderId(orderID);
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
	}

	@Override
	public void add(final String hostedDataID, final String orderID, final boolean declineHostedDataDuplicates, final TokenType tokenType,
			final Boolean assignToken) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setOrderId(orderID);
		item.setDeclineHostedDataDuplicates(Boolean.valueOf(declineHostedDataDuplicates));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
	}

	private static CreditCardData getCreditCardData(final CreditCard creditCard) {
		final CreditCardData creditCardData = new CreditCardData();
		creditCardData.setCardNumber(creditCard.cardNumber);
		creditCardData.setExpMonth(creditCard.expMonth);
		creditCardData.setExpYear(creditCard.expYear);
		creditCardData.setTrackData(creditCard.trackData);
		return creditCardData;
	}

	private static DEDirectDebitData getDEDirectDebitData(final DEDirectDebit account) {
		final DEDirectDebitData directDebitDE = new DEDirectDebitData();
		if (account.iban == null) {
			directDebitDE.setBankCode(account.bankCode);
			directDebitDE.setAccountNumber(account.accountNumber);
		} else {
			directDebitDE.setBIC(account.bic);
			directDebitDE.setIBAN(account.iban);
		}
		return directDebitDE;
	}

	@Override
	public void add(final String hostedDataID, final DEDirectDebit account, final TokenType tokenType, final Boolean assignToken, final String billingName) {
		final DataStorageItem item = getItem(hostedDataID);
		item.setDEDirectDebitData(getDEDirectDebitData(account));
		item.setTokenType(tokenType);
		item.setAssignToken(assignToken);
		item.setBillingName(billingName);
	}

}
