package de.firstdata.ipg.api.client.action.object;

import javax.xml.datatype.XMLGregorianCalendar;

import de.firstdata.ipg.api.client.action.Validate;
import de.firstdata.ipg.api.client.transaction.Amount;
import de.firstdata.ipg.api.client.transaction.creditcard.CreditCard;
import de.firstdata.ipg.api.client.transaction.customercard.CustomerCard;
import de.firstdata.ipg.api.client.transaction.dedirectdebit.DEDirectDebit;
import de.firstdata.ipg.api.schema.AdditionalRequestParameters;
import de.firstdata.ipg.api.schema.Billing;
import de.firstdata.ipg.api.schema.CreditCardData;
import de.firstdata.ipg.api.schema.CustomerCardData;
import de.firstdata.ipg.api.schema.DEDirectDebitData;
import de.firstdata.ipg.api.schema.GenderType;
import de.firstdata.ipg.api.schema.Payment;
import de.firstdata.ipg.api.schema.TransactionDetails;

public class ValidateObject extends IPGAPIActionObject implements Validate {

	public ValidateObject(final CreditCard card) {
		final CreditCardData data = new CreditCardData();
		data.setCardNumber(card.cardNumber);
		data.setExpMonth(card.expMonth);
		data.setExpYear(card.expYear);
		data.setCardCodeValue(card.cardCodeValue);
		data.setTrackData(card.trackData);
		getValidate().setCreditCardData(data);
	}

	public ValidateObject(final CustomerCard card) {
		final CustomerCardData data = new CustomerCardData();
		data.setCardNumber(card.cardNumber);
		data.setExpMonth(card.expMonth);
		data.setExpYear(card.expYear);
		getValidate().setCustomerCardData(data);
	}

	public ValidateObject(final DEDirectDebit account) {
		final DEDirectDebitData data = new DEDirectDebitData();
		if (account.iban == null) {
			data.setBankCode(account.bankCode);
			data.setAccountNumber(account.accountNumber);
		} else {
			data.setBIC(account.bic);
			data.setIBAN(account.iban);
		}
		data.setTrackData(account.trackData);
		getValidate().setDEDirectDebitData(data);
	}

	public ValidateObject(final String hostedDataID) {
		getPayment().getHostedDataID().add(hostedDataID);
	}

	@Override
	public void setHostedDataID(final String value) {
		getPayment().getHostedDataID().add(value);
	}

	@Override
	public void setHostedDataStoreID(final String value) {
		getPayment().setHostedDataStoreID(value);
	}

	@Override
	public void setStoreId(final String storeId) {
		this.validate.setStoreId(storeId);
	}

	@Override
	public void setPayment(final Amount amount) {
		final Payment payment = getPayment();
		payment.setChargeTotal(amount.total);
		payment.setCurrency(amount.currency);
	}

	@Override
	public void setAddress1(final String address1) {
		getBilling().setAddress1(address1);
	}

	@Override
	public void setAddress2(final String address2) {
		getBilling().setAddress2(address2);
	}

	@Override
	@Deprecated
	public void setAddressNumber(final String addressNumber) {
		getBilling().setAddrnum(addressNumber);
	}

	@Override
	public void setCity(final String city) {
		getBilling().setCity(city);
	}

	@Override
	public void setCompany(final String company) {
		getBilling().setCompany(company);
	}

	@Override
	public void setCountry(final String country) {
		getBilling().setCountry(country);
	}

	@Override
	public void setCustomerID(final String customerID) {
		getBilling().setCustomerID(customerID);
	}

	@Override
	public void setEmail(final String email) {
		getBilling().setEmail(email);
	}

	@Override
	public void setFax(final String fax) {
		getBilling().setFax(fax);
	}

	@Override
	public void setName(final String name) {
		getBilling().setName(name);
	}

	@Override
	public void setPhone(final String phone) {
		getBilling().setPhone(phone);
	}

	@Override
	public void setState(final String state) {
		getBilling().setState(state);
	}

	@Override
	public void setZip(final String zip) {
		getBilling().setZip(zip);
	}

	@Override
	public void setPersonalNumber(final String personalNumber) {
		getBilling().setPersonalNumber(personalNumber);
	}

	@Override
	public void setBirthDate(final XMLGregorianCalendar birthDate) {
		getBilling().setBirthDate(birthDate);
	}

	@Override
	public void setGender(final GenderType gender) {
		getBilling().setGender(gender);
	}

	@Override
	public void setFirstname(final String firstname) {
		getBilling().setFirstname(firstname);
	}

	@Override
	public void setSurname(final String surname) {
		getBilling().setSurname(surname);
	}

	@Override
	public void setStreetName(final String streetName) {
		getBilling().setStreetName(streetName);
	}

	@Override
	public void setHouseNumber(final String houseNumber) {
		getBilling().setHouseNumber(houseNumber);
	}

	@Override
	public void setHouseExtension(final String houseExtension) {
		getBilling().setHouseExtension(houseExtension);
	}

	@Override
	public void setMobilePhone(final String mobilePhone) {
		getBilling().setMobilePhone(mobilePhone);
	}

	@Override
	public void setInvoiceNumber(final String invoiceNumber) {
		getTransactionDetails().setInvoiceNumber(invoiceNumber);
	}

	@Override
	public void setDynamicMerchantName(final String dynamicMerchantName) {
		getTransactionDetails().setDynamicMerchantName(dynamicMerchantName);
	}

	@Override
	public void setPONumber(final String poNumber) {
		getTransactionDetails().setPONumber(poNumber);
	}

	@Override
	public void setIp(final String ip) {
		getTransactionDetails().setIp(ip);
	}

	@Override
	public void setOrderId(final String orderId) {
		getTransactionDetails().setOrderId(orderId);
	}

	@Override
	public void setTransactionOrigin(final String transactionOrigin) {
		getTransactionDetails().setTransactionOrigin(transactionOrigin);
	}

	@Override
	public de.firstdata.ipg.api.schema.Validate getValidate() {
		if (this.validate == null) {
			this.validate = new de.firstdata.ipg.api.schema.Validate();
		}
		return this.validate;
	}

	private Payment getPayment() {
		Payment payment = getValidate().getPayment();
		if (payment == null) {
			payment = new Payment();
			this.validate.setPayment(payment);
		}
		return payment;
	}

	private Billing getBilling() {
		Billing billing = this.validate.getBilling();
		if (billing == null) {
			billing = new Billing();
			this.validate.setBilling(billing);
		}
		return billing;
	}

	private TransactionDetails getTransactionDetails() {
		TransactionDetails transactionDetails = this.validate.getTransactionDetails();
		if (transactionDetails == null) {
			transactionDetails = new TransactionDetails();
			this.validate.setTransactionDetails(transactionDetails);
		}
		return transactionDetails;
	}

	@Override
	public void setAdditionalRequestParameters(final AdditionalRequestParameters additionalRequestParameters) {
		getTransactionDetails().setAdditionalRequestParameters(additionalRequestParameters);
	}

	@Override
	public void setIpgDeferredAuth(final Boolean isDeferredAuth) {
		getTransactionDetails().setIpgDeferredAuth(isDeferredAuth);
	}
}
