package de.firstdata.ipg.api.client.transaction.creditcard.object;

import javax.annotation.CheckForNull;

import de.firstdata.ipg.api.client.IPGApiClient;
import de.firstdata.ipg.api.client.transaction.Amount;
import de.firstdata.ipg.api.client.transaction.creditcard.CCCreditTransaction;
import de.firstdata.ipg.api.client.transaction.creditcard.CreditCard;
import de.firstdata.ipg.api.schema.BusinessApplicationIdentifierType;
import de.firstdata.ipg.api.schema.Secure3DAuthenticationRequest;
import de.firstdata.ipg.api.schema.Secure3DVerificationRequest;

/**
 * This class represents a <i>Credit</i> transaction and allows for setting its
 * optional attributes by calling the appropriate <code>setter</code> methods.
 * An instance of this class is created when calling
 * {@link IPGApiClient#createCreditTransaction(double, String, String, String)}.
 *
 * @author Andreas Schmid
 */
public final class CCCreditTransactionObject extends CCTransactionObject implements CCCreditTransaction {

	/**
	 * creates a <i>Credit</i> transaction with the minimum set of parameters -
	 * note that you can add optional fields by calling the appropriate
	 * <code>setter</code> methods on the returned
	 * {@link CCCreditTransactionObject} instance
	 *
	 */
	public CCCreditTransactionObject(final Amount amount, final CreditCard creditCard) {
		super("credit", amount, creditCard, null, null, null);
	}

	public CCCreditTransactionObject(final Amount amount, final String hostedDataID, final String hostedDataStoreID) {
		super("credit", amount, null, hostedDataID, hostedDataStoreID, null);
	}

	@Override
	public void setSecure3DVerificationRequest(final Secure3DVerificationRequest secure3dVerificationRequest) {
		getSecure3DRequest().setSecure3DVerificationRequest(secure3dVerificationRequest);
	}

	@Override
	public void setSecure3DAuthenticationRequest(final Secure3DAuthenticationRequest secure3dAuthenticationRequest) {
		getSecure3DRequest().setSecure3DAuthenticationRequest(secure3dAuthenticationRequest);
	}

	@Override
	public void setBusinessApplicationIdentifier(@CheckForNull final BusinessApplicationIdentifierType businessApplicationIdentifier) {
		getTransactionDetails().setBusinessApplicationIdentifier(businessApplicationIdentifier);
	}
}
